'''
 *category   Micropython application
 *package    MML Music box
 *author     Ichiro Shirasaka
 *copyright  2017 Ichiro Shirasaka All Rights Reserved
 *version    1.0'''

import os
import re
import math
from machine import Pin, PWM, SPI
import time

#SD Card initialize
def sddrv():
    import machine, sdcard
    spi = SPI(2, sck=Pin(18), mosi=Pin(23), miso=Pin(19))
    sd = sdcard.SDCard(spi, machine.Pin(5))
    os.mount(sd,'/sd')

#MML parameter 
tuning=440
def_tempo=120
def_octave=4
def_on_length=4
volume=8
tick_position = 0
note_diff = {"c":  -9, "c+": -8, "d-": -8, "d":  -7, "d+": -6,\
             "e-": -6, "e":  -5, "f":  -4, "f+": -3, "g-": -3,\
             "g":  -2, "g+": -1, "a-": -1, "a":   0, "a+":  1,\
             "b-": 1, "b": 2, "r": None}

note_pattern = re.compile(r"([a-g]|r)(\+|\#|\-)?(\d+)?(\.)?")
command_pattern = re.compile(r"(t|l|v|o)(\d+)?(\.)?")
octave_up_down_pattern = re.compile(r"(\<|\>)")

#Convert sequence
def cnv_seq(mml):
    global tuning,def_tempo,def_octave,def_on_length,volume,tick_position
    mml = mml.lower()
    sequence = list()
    read_position = 0
    pwm12 = PWM(Pin(12))  #SP Pin
    pwm12.duty(0)         #PWM duty = 0
    p2 = Pin(2, Pin.OUT)

    while read_position < len(mml):
        note_match = note_pattern.match(mml[read_position:])
        #Note process
        if note_match != None:
            print(("Note:%-4s" % note_match.group(0)), end="")
            note_code = note_match.group(1)
            accidential = note_match.group(2)
            on_length = note_match.group(3)
            period = note_match.group(4)
            # Get note length
            note_length = int(on_length) if on_length else def_on_length
            note_on_tick = (60.0 / def_tempo) * (4.0 / note_length)
            # note length * 1.5
            note_on_tick *= 1.5 if period else 1.0
            # Update sound end time
            #tick_position += note_on_tick
            # Calculate tone frequency
            if note_code == "r":
                note_frequency = 1
                pwm12.duty(0)
            else:
                pwm12.duty(128)
                note = note_diff[note_code]
                # Semitone process
                if accidential:
                    if accidential in "#+":
                        note += 1
                    elif accidential == "-":
                        note -= 1
                # Get Frequency(12temperament)
                note_frequency = tuning * (2 ** (def_octave - 4)) * (2 ** (note/ 12.0))
            pwm12.freq(int(round(note_frequency, 0)))
            p2.value(1)
            time.sleep_ms(int(note_on_tick*800))
            p2.value(0)
            time.sleep_ms(int(note_on_tick*200))
            print(("Freq:%4dHz " % int(note_frequency)), end="")
            print("Time:%.2fs" % note_on_tick)
            read_position += len(note_match.group(0))
            continue
        command_match = command_pattern.match(mml[read_position:])
        if command_match != None:
            print("CMD:%s" % command_match.group(0))
            command = command_match.group(1)
            value = command_match.group(2)
            period = command_match.group(3)
            if command == "t":
                #Tempo
                def_tempo = int(value)
            elif command == "v":
                #Volume
                def_volume = int(value)
            elif command == "l":
                #Default note length
                def_on_length = int(value)
                if period:
                    def_on_length *= 1.5
            elif command == "o":
                #Octave(absolute)
                def_octave = int(value)
            read_position += len(command_match.group(0))
            continue
        octave_match = octave_up_down_pattern.match(mml[read_position:])
        if octave_match != None:
            print("OCT:%s" % octave_match.group(0))
            #Octave(relative)
            if mml[read_position] == ">":
                def_octave += 1
            else:
                def_octave -= 1
            read_position += len(octave_match.group(0))
        else:
            read_position += 1
    pwm12.deinit()

#Music box main
sddrv()
while True:
    fl = os.listdir('sd/mml')
    print("*********MML FILE**************")
    for n in range(len(fl)):
        print("%d:%s" % (n + 1, fl[n]))
    print("*******************************")
    num = input("Select number:")
    fp = open('/sd/mml/'+fl[int(num) - 1], 'r')
    mml = fp.read()
    print("MML:%s" % mml)
    fp.close()
    cnv_seq(mml)

