'''
 *category   Micropython application
 *package    NTP CLOCK
 *author     Ichiro Shirasaka
 *copyright  2017 Ichiro Shirasaka All Rights Reserved
 *version    1.0'''
 
from machine import Pin
import time

LCD_WIDTH = 16  #LCD charactor number of 1 line
LCD_CHR = 1     #Display data
LCD_CMD = 0     #Command data

LCD_LINE_1 = 0x80   #Line 1
LCD_LINE_2 = 0xC0   #Line 2

# Define Pin number
pin_e  = Pin(26, Pin.OUT)
pin_rs = Pin(17, Pin.OUT)
pin_d4 = Pin(13, Pin.OUT)
pin_d5 = Pin(14, Pin.OUT)
pin_d6 = Pin(15, Pin.OUT)
pin_d7 = Pin(16, Pin.OUT)

#LCD Initialize
def lcd_init():
    lcd_byte(0x33, LCD_CMD)
    lcd_byte(0x32, LCD_CMD)
    lcd_byte(0x28, LCD_CMD)
    lcd_byte(0x0C, LCD_CMD)
    lcd_byte(0x06, LCD_CMD)
    lcd_byte(0x01, LCD_CMD)
    time.sleep_ms(1)

#LCD data out(nibble)
def lcd_nibble(bits):
    pin_d4.value((bits >> 4) & 1)
    pin_d5.value((bits >> 5) & 1)
    pin_d6.value((bits >> 6) & 1)
    pin_d7.value((bits >> 7) & 1)
    pin_e.value(1)
    pin_e.value(0)

#LCD data out(byte)
def lcd_byte(bits, mode):
    pin_rs.value(mode)
    lcd_nibble(bits)
    bits <<= 4;
    lcd_nibble(bits)

#LCD Put String
def lcd_string(message, line):
    lcd_byte(line, LCD_CMD)
    for c in message: 
        lcd_byte(ord(c), LCD_CHR)

#Convert local time
def localtime(esptime):
    global year,mon,day,hour,min,sec
    esptime +=9*3600        #GMT -> JST
    month1 = (31,28,31,30,31,30,31,31,30,31,30,31)
    month2 = (31,29,31,30,31,30,31,31,30,31,30,31)

    date = esptime // 86400
    time = esptime % 86400

    r1 = date % 1461
    y1 = date // 1461
    if r1 >= 366 :
        r2 = (r1-366) % 365
        y2 = (r1-366) // 365 + 1
        year = y1*4 + y2 + 2000
        for n in range(12):
            if  r2 - month1[n] < 0:
                mon = n+1
                break
            r2 -= month1[n]
        day = r2 + 1
    else:
        year = y1*4 + 2000
        for n in range(12):
            if  r1 - month2[n] < 0:
                mon = n+1
                break
            r1 -= month1[n] 
        day = r1
    hour = time // 3600
    min = (time % 3600) // 60
    sec = time % 3600 % 60

#calculate day of week
def wday(year, mon, day):
    week = "Sun","Mon","Tue","Wed","Thu","Fri","Sat"
    if(mon < 3 ):
        year -= 1
        mon += 12
    return week[int((year + year/4 - year/100 + year/400 + (13 * mon + 8)/5 + day)%7)]

try:
    import usocket as socket
except:
    import socket
try:
    import ustruct as struct
except:
    import struct

NTP_DELTA = 3155673600  #2000.1.1 0:0:0 
host = 'ntp.nict.jp'

#Connect by Station mode
def do_connect():
    import network
    sta_if = network.WLAN(network.STA_IF)
    if not sta_if.isconnected():
        print('connecting to network...')
        sta_if.active(True)
        sta_if.connect('39D45E92693CC5A51619F9CE57C6042F','eac5aa27eabe8a24a85f0543c4ad895b541f8d5e611e9b10ff9dbe384769b70')
        while not sta_if.isconnected():
            pass
    print('network config:', sta_if.ifconfig())

#Get time from NTP server
def ttime(): 
    NTP_QUERY = bytearray(48) 
    NTP_QUERY[0] = 0x1b 
    addr = socket.getaddrinfo(host, 123)[0][-1] 
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) 
    s.settimeout(1) 
    res = s.sendto(NTP_QUERY, addr) 
    msg = s.recv(48)
    s.close()
    val = struct.unpack("!I", msg[40:44])[0]
    return val - NTP_DELTA

#NTP Clock Main process
rtc_time = 0
do_connect()
rtc_time = ttime()
lcd_init()

while True:
    tick = time.ticks_ms()
    if  tick % 1000 == 0:
        rtc_time += 1
        localtime(rtc_time)
        lcd_string("%4d/%2d/%2d (%s)" % (year, mon, day, wday(year, mon, day)), LCD_LINE_1)
        lcd_string("    %2d:%02d:%02d" % (hour, min, sec), LCD_LINE_2)
        if sec == 0 and min == 0:
            rtc_time = ttime()
            print('get time')

